@@ -215,34 +215,35 @@ impl Node {
     }
 }
 
+/// Cursors are a bit weird, and they deserve an explanation.
+///
+/// Cursors express the location that an edit will happen. But because this is a skip list, when
+/// items are added or removed we need to not just splice in / remove elements, but also update:
+///
+/// - The next pointers of *previous* items
+/// - The index item. Each next pointer in a node names how many items are being "skipped over" by
+///   that pointer. Those "skipped over" counts need to be updated based on the change.
+///
+/// Anyway, to do all of this, a cursor names the item which *points to* the current location.
+///
+/// TODO: A cursor should store a PhantomData<Pin<&mut JumpRope>>.
 #[derive(Debug, Clone)]
 pub(crate) struct RopeCursor([SkipEntry; MAX_HEIGHT+1]);
 
 impl RopeCursor {
-    #[cfg(not(feature = "wchar_conversion"))]
-    fn update_offsets(&mut self, height: usize, by: isize) {
-        for i in 0..height {
-            unsafe {
-                // This is weird but makes sense when you realise the nexts in
-                // the cursor are pointers into the elements that have the
-                // actual pointers.
-                // Also adding a usize + isize is awful in rust :/
-                let skip = &mut (*self.0[i].node).nexts_mut()[i].skip_chars;
-                *skip = skip.wrapping_add(by as usize);
-            }
-        }
-    }
-    #[cfg(feature = "wchar_conversion")]
-    fn update_offsets_wchar(&mut self, height: usize, by_chars: isize, by_pairs: isize) {
+    fn update_offsets(&mut self, height: usize, by_chars: isize, #[cfg(feature = "wchar_conversion")] by_pairs: isize) {
         for i in 0..height {
             unsafe {
                 // This is weird but makes sense when you realise the nexts in
                 // the cursor are pointers into the elements that have the
                 // actual pointers.
+
                 // Also adding a usize + isize is awful in rust :/
                 let entry = &mut (*self.0[i].node).nexts_mut()[i];
                 entry.skip_chars = entry.skip_chars.wrapping_add(by_chars as usize);
-                entry.skip_pairs = entry.skip_pairs.wrapping_add(by_pairs as usize);
+                #[cfg(feature = "wchar_conversion")] {
+                    entry.skip_pairs = entry.skip_pairs.wrapping_add(by_pairs as usize);
+                }
             }
         }
     }
@@ -304,7 +305,7 @@ impl JumpRope {
     /// Note using this method in wasm significantly increases bundle size. Use
     /// [`new_with_seed`](Self::new_from_seed) instead.
     pub fn new() -> Self {
-        if cfg!(test) || cfg!(debug_assertions) {
+        if cfg!(test) || cfg!(debug_assertions) || !cfg!(feature = "ddos_protection") {
             Self::new_from_seed(123)
         } else {
             Self::new_from_entropy()
@@ -590,7 +591,7 @@ impl JumpRope {
             (*e).str.insert_in_gap(contents);
 
             #[cfg(feature = "wchar_conversion")] {
-                cursor.update_offsets_wchar(head_height, num_inserted_chars as isize, num_inserted_pairs as isize);
+                cursor.update_offsets(head_height, num_inserted_chars as isize, num_inserted_pairs as isize);
                 cursor.move_within_node(head_height, num_inserted_chars as isize, num_inserted_pairs as isize);
             }
             #[cfg(not(feature = "wchar_conversion"))] {
@@ -650,7 +651,7 @@ impl JumpRope {
             // .... aaaand update all the offset amounts.
 
             #[cfg(feature = "wchar_conversion")] {
-                cursor.update_offsets_wchar(head_height, num_inserted_chars as isize, num_inserted_pairs as isize);
+                cursor.update_offsets(head_height, num_inserted_chars as isize, num_inserted_pairs as isize);
                 cursor.move_within_node(head_height, num_inserted_chars as isize, num_inserted_pairs as isize);
             }
             #[cfg(not(feature = "wchar_conversion"))] {
@@ -679,7 +680,7 @@ impl JumpRope {
                 #[cfg(feature = "wchar_conversion")] {
                     num_end_pairs = (*e).num_surrogate_pairs() - (*e).str.gap_start_surrogate_pairs as usize;
                     debug_assert_eq!(num_end_pairs, count_utf16_surrogates(end_str));
-                    cursor.update_offsets_wchar(head_height, -(num_end_chars as isize), -(num_end_pairs as isize));
+                    cursor.update_offsets(head_height, -(num_end_chars as isize), -(num_end_pairs as isize));
                 }
                 #[cfg(not(feature = "wchar_conversion"))]
                 cursor.update_offsets(head_height, -(num_end_chars as isize));
@@ -1189,6 +1190,8 @@ impl JumpRope {
     }
 }
 
+/// These methods are only available if the `wchar_conversion` feature is enabled.
+#[cfg_attr(doc_cfg, doc(cfg(feature = "wchar_conversion")))]
 #[cfg(feature = "wchar_conversion")]
 impl JumpRope {
     /// Convert from a unicode character count to a wchar index, like what you'd use in Javascript,
@@ -1198,7 +1201,11 @@ impl JumpRope {
         cursor.wchar_pos(self.head.height)
     }
 
-    /// Convert a wchar index back to a unicode character count
+    /// Convert a wchar index back to a unicode character count.
+    ///
+    /// **NOTE:** This method's behaviour is undefined if the wchar offset is invalid. Eg, given a
+    /// rope with contents `𐆚` (a single character with wchar length 2), `wchars_to_chars(1)` is
+    /// undefined and may panic / change in future versions of diamond types.
     pub fn wchars_to_chars(&self, wchars: usize) -> usize {
         let cursor = self.cursor_at_wchar(wchars, true);
         cursor.global_char_pos(self.head.height)
@@ -1208,6 +1215,10 @@ impl JumpRope {
     /// This is compatible with NSString, Javascript, etc.
     ///
     /// Returns the insertion position in characters.
+    ///
+    /// **NOTE:** This method's behaviour is undefined if the wchar offset is invalid. Eg, given a
+    /// rope with contents `𐆚` (a single character with wchar length 2), `insert_at_wchar(1, ...)`
+    /// is undefined and may panic / change in future versions of diamond types.
     pub fn insert_at_wchar(&mut self, mut pos_wchar: usize, contents: &str) -> usize {
         pos_wchar = pos_wchar.min(self.len_wchars());
 
@@ -1225,6 +1236,10 @@ impl JumpRope {
 
     /// Remove items from the rope, specified by the passed range. The indexes are interpreted
     /// as wchar offsets (like you'd get in javascript / C# / etc).
+    ///
+    /// **NOTE:** This method's behaviour is undefined if the wchar offset is invalid. Eg, given a
+    /// rope with contents `𐆚` (a single character with wchar length 2), `remove_at_wchar(1..2)`
+    /// is undefined and may panic / change in future versions of diamond types.
     pub fn remove_at_wchar(&mut self, mut range: Range<usize>) {
         range.end = range.end.min(self.len_wchars());
         if range.is_empty() { return; }
@@ -1245,8 +1260,14 @@ impl JumpRope {
     }
 
     /// Replace the characters in the specified wchar range with content.
+    ///
+    /// **NOTE:** This method's behaviour is undefined if the wchar offset is invalid. Eg, given a
+    /// rope with contents `𐆚` (a single character with wchar length 2),
+    /// `replace_at_wchar(1..2, ...)` is undefined and may panic / change in future versions of
+    /// diamond types.
     pub fn replace_at_wchar(&mut self, range: Range<usize>, content: &str) {
-        // This is not optimized.
+        // TODO: Optimize this. This method should work similarly to replace(), where we create
+        // a single cursor and use it in both contexts.
         if !range.is_empty() {
             self.remove_at_wchar(range.clone());
         }
