@@ -6,7 +6,7 @@ use rand::prelude::*;
 use std::cmp::min;
 use std::ops::Range;
 
-const UCHARS: [char; 23] = [
+const UNI_CHARS: [char; 23] = [
   'a', 'b', 'c', '1', '2', '3', ' ', '_', // ASCII
   '©', '¥', '½', // The Latin-1 suppliment (U+80 - U+ff)
   'Ύ', 'Δ', 'δ', 'Ϡ', // Greek (U+0370 - U+03FF)
@@ -17,18 +17,19 @@ const UCHARS: [char; 23] = [
 fn random_unicode_string(len: usize, rng: &mut SmallRng) -> String {
     let mut s = String::new();
     for _ in 0..len {
-        s.push(UCHARS[rng.gen_range(0 .. UCHARS.len())] as char);
+        s.push(UNI_CHARS[rng.gen_range(0 .. UNI_CHARS.len())] as char);
     }
     s
 }
 
-const CHARS: &[u8; 83] = b" ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*()[]{}<>?,./";
+const ASCII_CHARS: &[u8; 83] = b" ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*()[]{}<>?,./";
 
 // Gross. Find a way to reuse the code from random_unicode_string.
+#[allow(unused)]
 fn random_ascii_string(len: usize, rng: &mut SmallRng) -> String {
     let mut s = String::new();
     for _ in 0..len {
-        s.push(CHARS[rng.gen_range(0 .. CHARS.len())] as char);
+        s.push(ASCII_CHARS[rng.gen_range(0 .. ASCII_CHARS.len())] as char);
     }
     s
 }
@@ -128,19 +129,20 @@ fn really_long_ascii_string() {
     let mut rng = SmallRng::seed_from_u64(1234);
     let len = 2000;
     let s = random_ascii_string(len, &mut rng);
+    // let s = random_unicode_string(len, &mut rng);
 
     let mut r = JumpRope::from(s.as_str());
     check(&r, s.as_str());
 
     // Delete everything but the first and last characters
     r.remove(1..len - 1);
-    let expect = format!("{}{}", s.as_bytes()[0] as char, s.as_bytes()[len-1] as char);
+    let expect = format!("{}{}", s.chars().next().unwrap(), s.chars().rev().next().unwrap());
     check(&r, expect.as_str());
 }
 
 
 use std::ptr;
-use jumprope::JumpRope;
+use jumprope::{JumpRope, JumpRopeBuf};
 
 fn string_insert_at(s: &mut String, char_pos: usize, contents: &str) {
     // If you try to write past the end of the string for now I'll just write at the end.
@@ -332,4 +334,70 @@ fn fuzz_wchar_forever() {
         if seed % 100 == 0 { println!("seed: {seed}"); }
         random_edits_wchar(seed, false);
     }
+}
+
+
+fn random_edits_buffered(seed: u64, verbose: bool) {
+    let mut r = JumpRopeBuf::new();
+    let mut s = String::new();
+
+    // let mut rng = rand::thread_rng();
+    let mut rng = SmallRng::seed_from_u64(seed);
+
+    for _i in 0..400 {
+    // for _i in 0..19 {
+        if verbose { println!("{_i} s: '{s}'"); }
+        // r.print();
+
+        let len = s.chars().count();
+
+        // if _i == 1 {
+        //     println!("haaayyy");
+        // }
+        // println!("i {}: {}", i, len);
+
+        if len == 0 || (len < 1000 && rng.gen::<f32>() < 0.5) {
+            // Insert.
+            let pos = rng.gen_range(0..len+1);
+            // Sometimes generate strings longer than a single node to stress everything.
+            let text = random_unicode_string(rng.gen_range(0..20), &mut rng);
+            if verbose {
+                println!("Inserting '{text}' at char {pos} (Byte length: {}, char len: {}, wchar len: {})",
+                         text.len(), text.chars().count(),
+                         text.chars().map(|c| c.len_utf16()).sum::<usize>()
+                );
+            }
+
+            r.insert(pos, text.as_str());
+            string_insert_at(&mut s, pos, text.as_str());
+        } else {
+            // Delete
+            let pos = rng.gen_range(0..len);
+            let dlen = min(rng.gen_range(0..10), len - pos);
+            if verbose {
+                println!("Removing {dlen} characters at {pos}");
+            }
+
+            r.remove(pos..pos+dlen);
+            string_del_at(&mut s, pos, dlen);
+        }
+        // dbg!(&r);
+    }
+
+    let rope = r.into_inner();
+    check(&rope, s.as_str());
+}
+
+#[test]
+fn fuzz_buffered_once() {
+    random_edits_buffered(0, false);
+}
+
+#[test]
+#[ignore]
+fn fuzz_buffered_forever() {
+    for seed in 0.. {
+        if seed % 1000 == 0 { println!("seed: {seed}"); }
+        random_edits_buffered(seed, false);
+    }
 }
\ No newline at end of file
